// Authors: Shoma Berkemeyer, Julius Schöning
// Citation: please cite the paper 'Feeling Hungry --- Association of Dietary Patterns with Food Choices Using Scene Perception' 2020 by the authors

#include "datastate.h"

#include <QCoreApplication>

#include "tobii.h"
#include "tobii_streams.h"
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "globalobjects.h"

void url_receiver ( char const* url, void* user_data )
{
    char* buffer = (char*)user_data;
    if( *buffer != '\0' ) return; // only keep first value

    if( strlen( url ) < 256 )
        strcpy( buffer, url );
}

   void gaze_point_callback( tobii_gaze_point_t const* gaze_point, void* /* user_data */)
    {
        float position_x=424242;
        float position_y=424242;
        // Check that the data is valid before using it
        quint32 windowWidth = 1920;
        quint32 windowHieght = 1080;
        if(g_mainWindow != nullptr && g_state != nullptr){
            windowWidth = g_mainWindow->size().width();
            windowHieght = g_mainWindow->size().height();
        if( gaze_point->validity == TOBII_VALIDITY_VALID ){
            position_x = gaze_point->position_xy[ 0 ]*windowWidth;
            position_y = gaze_point->position_xy[ 1 ]*windowHieght;
            g_state->gazeSetPointsX.append(qRound(position_x));
            g_state->gazeSetPointsY.append(qRound(position_y));
            //qDebug()<<position_x;
            if(g_state->gazeSetPointsX.size()>5)
                g_state->gazeSetPointsX.removeFirst();
            if(g_state->gazeSetPointsY.size()>5)
                g_state->gazeSetPointsY.removeFirst();
        }
        else {
            position_x = 424242;
            position_y = 424242;
        }
        quint32 timestamp = QDateTime::currentMSecsSinceEpoch() - QDateTime(QDate::currentDate()).toMSecsSinceEpoch();
        g_state->gPListTime.append(timestamp);
        g_state->gPListX.append(qRound(position_x));
        g_state->gPListY.append(qRound(position_y));
        }
        else
            return;
    }

DataState::DataState(QObject *parent) :
    QObject(parent),
    participantID(QStringLiteral("P%1").arg(QDateTime(QDateTime::currentDateTime()).toString("ddmmhh"))),
    currentImage(0)
{

}

DataState::~DataState()
{

}


void DataState::startNewSurvey()
{
    qDebug("start new survey");
    emit SignalStartNewSurvey();
}

void DataState::continueSurvey()
{
    qDebug("continue survey");
    emit SignalContinueSurvey();
}

void DataState::updateImage()
{
    qDebug("update Image");
    emit SignalUpdateImage();
}

void DataState::startEyeTracking()
{
    qDebug("Test Start Eyetracking");
    if(g_state->ContinueForm){
        QString tmp = g_mainWindow->enterPatID->text();
        if(!tmp.isNull()){
            if (tmp.size()!=7){
                emit SignalContinueSurvey();
                return;
            }
            else {
            g_state->participantID=g_mainWindow->enterPatID->text()+"_cont";
            qDebug()<<g_state->participantID;
            g_state->answeredDish=g_mainWindow->dish->currentText();
            }
        }
    }
    else{
       g_state->answeredBirthday=g_mainWindow->integerSpinBox->value();
       g_state->answeredSex=g_mainWindow->sex->currentText();
       g_state->answeredCountrie=g_mainWindow->countr->currentText();
       g_state->answeredMensaType=g_mainWindow->mensa->currentText();
       g_state->answeredFoodType=g_mainWindow->cb->currentText();
       g_state->answeredDish=g_mainWindow->dish->currentText();
    }
    qDebug("StartingThreads");
    if(!futureGazeTrackingSet){
        futureGazeTracking= QtConcurrent::run(this,&DataState::gazeTrackingThread);
        futureGazeTrackingSet=true;
    }

    if(!futureGazeDrawingSet){
        futureGazeDrawing = QtConcurrent::run(this,&DataState::gazeDrawingThread);
        futureGazeDrawingSet=true;
    }
    emit SignalStartEyeTracking();
}

void DataState::startSlideShow()
{
    g_mainWindow->removeAction(g_mainWindow->key);
    g_state->GazeSettingStop=true;

    emit SignalStartSlideShow();
}

void DataState::startSlideShowThread(){
    if(g_state->gazeSetPointsX.last()!=424242 && g_state->gazeSetPointsY.last()!=424242){
     g_state->gazeOffSetX = qint64(g_state->gazeSetPointsX.last())-qint64(g_mainWindow->size().width()/2);
     g_state->gazeOffSetY = qint64(g_state->gazeSetPointsY.last())-qint64(g_mainWindow->size().height()/2);
    }
    if(!futureSlideShowSet){
    futureSlideShow = QtConcurrent::run(this,&DataState::slideShowThread);
    futureSlideShowSet=true;
    }
}


void DataState::endSlideShow()
{
    qDebug("Test End Slide Show");
    emit SignalEndSlideShow();
}

void DataState::delay(int sec, int msec)
{
    QTime dieTime= QTime::currentTime().addSecs(sec).addMSecs(msec);
    while (QTime::currentTime() < dieTime)
        QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
}

void DataState::slideShowThread(){
    qDebug("Thread3");

    for(int x=1; x < g_state->imageList.length() && !StopSlideShow; x++)
    {
           currentImage=x;
           updateImage();
           qDebug()<<x;
           if(x==1 || x==g_state->imageList.length()-1)
               delay(2,0);
           else
               delay(10, 0);
    }
    StopSlideShow=true;
    g_state->endSlideShow();

}

void DataState::gazeDrawingThread(){
    qDebug("Thread2");
    screenUpdateTime = QDateTime::currentMSecsSinceEpoch() - QDateTime(QDate::currentDate()).toMSecsSinceEpoch();
    while(!this->GazeSettingStop)
     {
        quint32 currentTime = QDateTime::currentMSecsSinceEpoch() - QDateTime(QDate::currentDate()).toMSecsSinceEpoch();
        if(g_state->gazeSetPointsY.size()>4 && g_state->gazeSetPointsX.size()>4 && (currentTime-screenUpdateTime)>66){
              //    qDebug("SignalUpdateGazeOnScreen");
            //g_mainWindow->sceneFixation->isActive()
            emit g_state->SignalUpdateGazeOnScreen();
        }


    }
    qDebug("Kill Thread2");

}

void DataState::gazeTrackingThread(){
    qDebug("Thread1");
    qDebug("start eyex");
               // Create API
                   tobii_api_t* api = NULL;
                   tobii_error_t result = tobii_api_create( &api, NULL, NULL );
                  assert( result == TOBII_ERROR_NO_ERROR );

                  // Enumerate devices to find connected eye trackers, keep the first
                  char url[ 256 ] = { 0 };
                   result = tobii_enumerate_local_device_urls( api, url_receiver, url );
                   qDebug()<<result;
                   assert( result == TOBII_ERROR_NO_ERROR );
                   if(*url == '\0')
                   {
                       qDebug("Error: No device found");
                       this->GazeDeviceFail = true;
                       this->GazeDeviceInit = true;
                       return;
                   }
                   qDebug("device found");
                  // Connect to the first tracker found
                  tobii_device_t* device = NULL;
                   result = tobii_device_create( api, url, TOBII_FIELD_OF_USE_INTERACTIVE, &device );
                   assert( result == TOBII_ERROR_NO_ERROR );

                   // Subscribe to gaze data
                   result = tobii_gaze_point_subscribe( device, gaze_point_callback, 0 );
                   assert( result == TOBII_ERROR_NO_ERROR );
                   // This sample will collect 1000 gaze points

                   g_state->GazeTrackingStart=true;
                   g_state->GazeDeviceInit=true;
               while(!this->GazeTrackingStop)
                  {

                      // Optionally block this thread until data is available. Especially useful if running in a separate thread.
                      result = tobii_wait_for_callbacks( 1, &device );
                      assert( result == TOBII_ERROR_NO_ERROR || result == TOBII_ERROR_TIMED_OUT );

                    // Process callbacks on this thread if data is available
                  result = tobii_device_process_callbacks( device );
                    assert( result == TOBII_ERROR_NO_ERROR );
              }
               g_state->GazeTrackingStart=false;
               g_state->GazeDeviceInit=false;

                  // Cleanup
                    result = tobii_gaze_point_unsubscribe( device );
                    assert( result == TOBII_ERROR_NO_ERROR );
                  result = tobii_device_destroy( device );
                   assert( result == TOBII_ERROR_NO_ERROR );
                  result = tobii_api_destroy( api );
                 assert( result == TOBII_ERROR_NO_ERROR );
               qDebug("Kill Thread1");
                return;
}
