// Authors: Shoma Berkemeyer, Julius Schöning
// Citation: please cite the paper 'Feeling Hungry --- Association of Dietary Patterns with Food Choices Using Scene Perception' 2020 by the authors

#ifndef DATASTATE_H
#define DATASTATE_H

#include <QObject>
#include <QList>
#include <QSignalMapper>
#include <QtConcurrent/QtConcurrent>


class DataState : public QObject
{
    Q_OBJECT
public:
    enum ChangeFlags {
        CurrentObjectChange = 1<<0,
        CurrentFrameChange = 1<<1,
        ObjectsListChange = 1<<2
    };

    explicit DataState(QObject *parent = 0);



    ~DataState();

    void delay(int sec, int msec);

    QString dataStateVersion;
    QString participantID;
    bool SetInitForm = false;
    bool ContinueForm = false;
    bool StartSlideShow = false;
    bool StopSlideShow =false;
    bool GazeDeviceInit = false;
    bool GazeDeviceFail = false;
    bool GazeTrackingStart =false;
    bool GazeTrackingStop = false;
    bool GazeSettingStop = false;

    bool futureSlideShowSet = false;
    QFuture<void> futureSlideShow;
    bool futureGazeTrackingSet = false;
    QFuture<void> futureGazeTracking;
    bool futureGazeDrawingSet = false;
    QFuture<void> futureGazeDrawing;

    int currentImage;
    QList<QString> imageList = QList<QString>() <<QString(":/images/fixation.png")
           << QString(":/images/dish_white.png")
           << QString(":/images/dish1.png")
           << QString(":/images/dish2.png")
           << QString(":/images/dish3.png")
           << QString(":/images/dish4.png")
           << QString(":/images/dish5.png")
           << QString(":/images/fixation.png");


    QList<quint32> imageChangeListTime;
    QList<QString> imageChangeListName;
    QList<quint32> gPListTime;
    QList<quint32> gPListX;
    QList<quint32> gPListY;
    QList<quint32> gazeSetPointsX;
    QList<quint32> gazeSetPointsY;
    qint64 gazeOffSetX = 0;
    qint64 gazeOffSetY = 0;
    quint32 screenUpdateTime = 0;
    quint16 answeredBirthday;
    QString answeredSex;
    QString answeredMensaType;
    QString answeredCountrie;
    QString answeredFoodType;
    QString answeredDish;



public slots:
    void startNewSurvey();//new
    void continueSurvey();//new
    void updateImage();//new
    void startEyeTracking();//new
    void startSlideShow();
    void startSlideShowThread();
    void endSlideShow();//new
    void slideShowThread();
    void gazeDrawingThread();
    void gazeTrackingThread();





signals:
    void SignalStartNewSurvey();
    void SignalContinueSurvey();
    void SignalStartEyeTracking();
    void SignalUpdateGazeOnScreen();
    void SignalStartSlideShow();
    void SignalEndSlideShow();
    void SignalUpdateImage();

};

#endif // DATASTATE_H
