// Authors: Shoma Berkemeyer, Julius Schöning
// Citation: please cite the paper 'Feeling Hungry --- Association of Dietary Patterns with Food Choices Using Scene Perception' 2020 by the authors

#include "mainwindow.h"

#include <QDebug>
#include <QGraphicsScene>
#include <QMenu>
#include <QFileDialog>
#include <QApplication>
#include <QFile>
#include <QMessageBox>
#include <QObject>
#include <QGroupBox>
#include <QPainter>
#include <QScreen>
#include <QGraphicsView>
#include <QGraphicsSimpleTextItem>


#include "globalobjects.h"




MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags) :
    QMainWindow(parent, flags)
{
    QFont font = this->font();
      font.setPointSize(20);
      this->setFont(font);

    // ==== SETUP UI ====
    if(g_state->SetInitForm==false){
        g_state->SetInitForm=true;
        //onActionStartSurvey();
        onActionNew();
    }
    else{
        qDebug("else");
    }


    setWindowFlag(Qt::FramelessWindowHint);
    showFullScreen();

    registerSig();


}

MainWindow::~MainWindow()
{

}

void MainWindow::registerSig(){
    QObject::connect(g_state, SIGNAL(SignalStartNewSurvey()),this, SLOT(onActionStartSurvey()));
    QObject::connect(g_state, SIGNAL(SignalContinueSurvey()),this, SLOT(onActionContinueSurvey()));


    QObject::connect(g_state, SIGNAL(SignalStartEyeTracking()),this, SLOT(onActionSetUpEyeTracker()));
    QObject::connect(g_state, SIGNAL(SignalUpdateGazeOnScreen()),this, SLOT(onActionDrawGazePoint()));

    QObject::connect(g_state, SIGNAL(SignalStartSlideShow()),this, SLOT(onActionStartSlideShow()));
    QObject::connect(g_state, SIGNAL(SignalUpdateImage()),this, SLOT(onActionShowImage()));


    QObject::connect(g_state, SIGNAL(SignalEndSlideShow()),this, SLOT(onActionSaveResults()));
}


void MainWindow::onActionRest() //new
{
    g_state =  new DataState();
    registerSig();
    qDebug()<<g_state->participantID;
    qDebug()<<g_state->GazeDeviceFail;
    g_state->SetInitForm=true;
    onActionNew();
    return;
}

void MainWindow::onActionNew()
{
    welcomeLabel = new QLabel();
    welcomeLabel->setText("Willkommen zu Eye Tracking Studie");
    welcomeLabel->setAlignment(Qt::AlignHCenter);

    //setCentralWidget(welcomeLabel);


       horizontalGroupBox = new QGroupBox(tr(""));
       horizontalGroupBox->setFlat(true);
       QHBoxLayout *layout = new QHBoxLayout;

       buttonsFirstRun = new QPushButton(tr("Neuer Teilnehmer"));
       QObject::connect(buttonsFirstRun, SIGNAL(clicked()),g_state, SLOT(startNewSurvey()));

       buttonsSecondRun = new QPushButton(tr("Zweiter Durchlauf"));
       QObject::connect(buttonsSecondRun, SIGNAL(clicked()),g_state, SLOT(continueSurvey()));


       layout->addWidget(buttonsFirstRun);
       layout->addWidget(buttonsSecondRun);

       horizontalGroupBox->setLayout(layout);


       initGroup = new QGroupBox(tr(""));
          initForm = new QFormLayout;
          initForm ->addRow(welcomeLabel);
          initForm ->addRow(horizontalGroupBox);

       initGroup->setLayout(initForm);
       initGroup->setFlat(true);
       initGroup->setFixedSize(
                          1000,
                          400);
       initGroup->setAlignment(Qt::AlignCenter);

       QHBoxLayout *layout2 = new QHBoxLayout;
        layout2->addWidget(initGroup);
       layout2->setAlignment(Qt::AlignCenter);

           QWidget *window = new QWidget();
           window->setLayout(layout2);


           setCentralWidget(window);
    return;
}

void MainWindow::onActionStartSurvey()
{
    qDebug("startsurvey");
    welcomeLabel = new QLabel();
    welcomeLabel->setText("Einige Fragen zum Einstieg");
    welcomeLabel->setAlignment(Qt::AlignHCenter);

    QLabel* space = new QLabel();
    space->setText("");
    space->setAlignment(Qt::AlignHCenter);

    QLabel* patID = new QLabel();
    patID->setText(g_state->participantID);



    integerSpinBox = new QSpinBox;
    integerSpinBox->setRange(1902, 2002);
    integerSpinBox->setSingleStep(1);
    integerSpinBox->setValue(2002);

    sex =new QComboBox();
    sex->addItem("weiblich");
    sex->addItem("männlich");
    sex->addItem("divers");

    mensa =new QComboBox();
    mensa->addItem("Student");
    mensa->addItem("Mitarbeiter");
    mensa->addItem("Besucher");

    countr =new QComboBox();
    foreach(QString item, countries)
          countr->addItem(item);

    cb =new QComboBox();
    cb->addItem("Mischkostler");
    cb->addItem("Vegetarier");
    cb->addItem("Veganer");
    cb->addItem("Vergetarier mit Fisch");
    cb->addItem("Paleo-Ernährung");
    cb->addItem("Rohköstler");
    cb->addItem("Vollwerternährung");
    cb->addItem("Frutarier");
    cb->addItem("anderen");

    dish =new QComboBox();
    foreach(QString item, dishes)
          dish->addItem(item);
    dish->addItem("-noch nicht entschieden-");


    startTracking = new QPushButton(tr("&Start Eye-Tracking"));
    startTracking->setDefault(false);
    QObject::connect(startTracking, SIGNAL(clicked()),g_state, SLOT(startEyeTracking()));


    initGroup = new QGroupBox(tr(""));
       initForm = new QFormLayout;
       initForm ->addRow(welcomeLabel);
       initForm ->addRow(space);
       initForm ->addRow(new QLabel(tr("Teilnehmer ID:")), patID);
       initForm ->addRow(space);
       initForm ->addRow(new QLabel(tr("Geburtsjahr:")), integerSpinBox);
       initForm ->addRow(new QLabel(tr("Gender:")), sex);
       initForm ->addRow(new QLabel(tr("Mensa-Type:")), mensa);
       initForm ->addRow(new QLabel(tr("Herkunftsland:")), countr);
       initForm ->addRow(space);
       initForm ->addRow(new QLabel(tr("Ernährungsstil:           ")));
       initForm ->addRow(cb);
       initForm ->addRow(space);
       initForm ->addRow(new QLabel(tr("Was essen Sie heute?")));
       initForm ->addRow(dish);
       initForm ->addRow(space);
       initForm ->addRow(startTracking);

    initGroup->setLayout(initForm);
    initGroup->setFlat(true);
    initGroup->setFixedSize(1200,1080);

    QHBoxLayout *layout = new QHBoxLayout;
     layout->addWidget(initGroup);
    layout->setAlignment(Qt::AlignCenter);


        QWidget *window = new QWidget();
        window->setLayout(layout);

        setCentralWidget(window);
        qDebug("form");
    return;


 }
void MainWindow::onActionContinueSurvey()
{
    g_state->ContinueForm=true;
    welcomeLabel = new QLabel();
    welcomeLabel->setText("Willkommen zurück");
    welcomeLabel->setAlignment(Qt::AlignHCenter);

    QLabel* space = new QLabel();
    space->setText("");
    space->setAlignment(Qt::AlignHCenter);

    enterPatID = new QLineEdit();

    dish =new QComboBox();
    foreach(QString item, dishes)
          dish->addItem(item);
    dish->addItem("-ein anderes Gericht-");


    startTracking = new QPushButton(tr("&Start Eye-Tracking"));
    startTracking->setDefault(false);
    QObject::connect(startTracking, SIGNAL(clicked()),g_state, SLOT(startEyeTracking()));


    initGroup = new QGroupBox(tr(""));
       initForm = new QFormLayout;
       initForm ->addRow(welcomeLabel);
       initForm ->addRow(space);
       initForm ->addRow(new QLabel(tr("Ihre Teilnehmer ID:")), enterPatID);
       initForm ->addRow(space);
       initForm ->addRow(space);
       initForm ->addRow(new QLabel(tr("Was aßen Sie heute?")));
       initForm ->addRow(dish);
       initForm ->addRow(space);
       initForm ->addRow(startTracking);

    initGroup->setLayout(initForm);
    initGroup->setFlat(true);
    initGroup->setFixedSize(
                800,
                1000);

    QHBoxLayout *layout = new QHBoxLayout;
     layout->addWidget(initGroup);
    layout->setAlignment(Qt::AlignCenter);


        QWidget *window = new QWidget();
        window->setLayout(layout);

        setCentralWidget(window);
        qDebug("form");
    return;

}


void MainWindow::onActionStartSlideShow()
{
    viewFixation->resetCachedContent();
    sceneFixation->clear();
    qDebug("OnSlideShow");
    if(g_state->SetInitForm==true && g_state->StartSlideShow==false){
        g_state->StartSlideShow=true;
        qDebug("OnSlideShowInIf");

       g_state->startSlideShowThread();
    }
    return;
}



void MainWindow::onActionSetUpEyeTracker()
{
    qDebug("waitForInitBegin");
    while(g_state->gPListX.empty())
     {
  //      qDebug("waitForInit");
     }

    if(g_state->GazeDeviceFail){
         g_state->endSlideShow();
         return;
     }

     // wait until the device is registered
     while(!g_state->GazeTrackingStart)
     {
         qDebug("wait");
     }

    qDebug("set up eye tracker");

    QString filename = g_state->imageList.first();
    QLabel* lbl = new QLabel(this);
    lbl->setAlignment(Qt::AlignCenter);
     QPixmap pix;
    if(pix.load(filename)){
        /** scale pixmap to fit in label'size and keep ratio of pixmap */
        pix = pix.scaled(this->size(),Qt::KeepAspectRatio);
        lbl->setPixmap(pix);
    }

    sceneFixation = new QGraphicsScene(0,0,size().width(),size().height(), this);
    sceneFixation->setBackgroundBrush(pix);
    QGraphicsSimpleTextItem * interactionText = sceneFixation->addSimpleText(QString("Wenn die Blickpunkte die Mitte fixieren können,"));
    interactionText->setBrush(Qt::red);
    interactionText->setFont(QFont("Arial", 30));
    QRectF bR = interactionText->sceneBoundingRect();
    interactionText->setPos( size().width()/2 - bR.width()/2, size().height() - qRound(2.5*bR.height()) );

    QGraphicsSimpleTextItem * interactionText1 = sceneFixation->addSimpleText(QString("mit der Taste 'C' die Studie starten."));
    interactionText1->setBrush(Qt::red);
    interactionText1->setFont(QFont("Arial", 30));
    bR = interactionText1->sceneBoundingRect();
    interactionText1->setPos( size().width()/2 - bR.width()/2, size().height() - qRound(1.5*bR.height()) );

    gazeEllipse =  sceneFixation->addEllipse(-50,-50,25,25);
    gazeEllipse->setBrush(Qt::yellow);

    viewFixation = new QGraphicsView;
    viewFixation->setScene(sceneFixation);
    viewFixation->setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
    viewFixation->setVerticalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
    setCentralWidget(viewFixation);


    key = new QAction(this);
    key->setShortcut(Qt::Key_C);
    QObject::connect(key, SIGNAL(triggered()), g_state, SLOT(startSlideShow()));
    this->addAction(key);

}

void MainWindow::onActionDrawGazePoint()
{
    viewFixation->resetCachedContent();
    if(!g_state->StartSlideShow){
    QRectF bR = gazeEllipse->sceneBoundingRect();
       if(g_state->gazeSetPointsX.size()>4 && g_state->gazeSetPointsY.size()>4)
        {
           if(g_state->gazeSetPointsX.last()!=424242 && g_state->gazeSetPointsY.last()!=424242){
           gazeEllipse->setPos( qRound((g_state->gazeSetPointsX.last()) - bR.width()/2), qRound((g_state->gazeSetPointsY.last()) -  bR.height()/2) );
           }
           }
    }
}

void MainWindow::onActionShowImage()
{
    if( g_state->StartSlideShow==true){


        QString filename = g_state->imageList.at(g_state->currentImage);
        QLabel* lbl = new QLabel(this);
        lbl->setAlignment(Qt::AlignCenter);
         QPixmap pix;
        if(pix.load(filename)){
            /** scale pixmap to fit in label'size and keep ratio of pixmap */
            pix = pix.scaled(this->size(),Qt::KeepAspectRatio);
            lbl->setPixmap(pix);
        }
        viewFixation->resetCachedContent();
        sceneFixation->setBackgroundBrush(pix);
//        setCentralWidget(lbl);
        qint32 timestamp = QDateTime::currentMSecsSinceEpoch() - QDateTime(QDate::currentDate()).toMSecsSinceEpoch();
        g_state->imageChangeListTime.append(timestamp);
        QString imageChangeItem = QStringLiteral("Img%2").arg(g_state->currentImage);
        g_state->imageChangeListName.append(imageChangeItem);
    }

}

void MainWindow::onActionSaveResults()
{

    cleanLabel = new QLabel();
    setCentralWidget(cleanLabel);
    g_state->GazeTrackingStop=true;
    g_state->GazeSettingStop=true;
    g_state->StopSlideShow=true;
    g_state->futureGazeDrawing.~QFuture();
    g_state->futureGazeTracking.~QFuture();
    QFile file(g_state->participantID+".txt");
    if (!file.open(QIODevice::WriteOnly)) {
        qDebug() << "Cannot open file for writing: ";
        qDebug() << qPrintable(file.errorString());
        return;
    }

   QTextStream out(&file);

   if(!g_state->gPListX.empty() && !g_state->imageChangeListTime.empty()){
   quint32 timeSlideShowStart = g_state->imageChangeListTime.first();
   QString imStr = g_state->imageChangeListName.first();

    for(int x=0;x<g_state->gPListX.size()-1;x++){
        qint64 time=qint64(g_state->gPListTime.at(x)) - qint64(timeSlideShowStart);
        if( time>0){
            qint64 gPx = -100;
            qint64 gPy = -100;
            if(g_state->gPListX.at(x)!=424242)
                gPx=qint64(g_state->gPListX.at(x)-g_state->gazeOffSetX);
            if(g_state->gPListY.at(x)!=424242)
                gPy=qint64(g_state->gPListY.at(x)-g_state->gazeOffSetY);
            QString gazeItem = QStringLiteral("%1;%2;%3;").arg(time).arg(gPx).arg(gPy);
            if(!g_state->imageChangeListName.empty()){
            if(g_state->gPListTime.at(x)>g_state->imageChangeListTime.first()){
               imStr = g_state->imageChangeListName.takeFirst();
               g_state->imageChangeListTime.removeFirst();
            }
            }
            out << gazeItem+imStr  << endl;
        }
    }
   }
    file.close();

    QFile file1(g_state->participantID+"_info.txt");
    if (!file1.open(QIODevice::WriteOnly)) {
        qDebug() << "Cannot open file for writing: ";
        qDebug() << qPrintable(file.errorString());
        return;
    }
   QTextStream out1(&file1);
   QString addInfo = QStringLiteral("ID;%1").arg(g_state->participantID.left(7));
   out1<<addInfo <<endl;
   addInfo = QStringLiteral("gazeOffSetX;%1").arg(g_state->gazeOffSetX);
   out1<<addInfo <<endl;
   addInfo = QStringLiteral("gazeOffSetY;%1").arg(g_state->gazeOffSetY);
   out1<<addInfo <<endl;
   g_state->gazeOffSetY = g_state->gazeSetPointsY.last();
   if(!g_state->ContinueForm){
        addInfo = QStringLiteral("Geburtsjahr;%1").arg(g_state->answeredBirthday);
        out1<<addInfo <<endl;
        addInfo = QStringLiteral("Gender;%1").arg(g_state->answeredSex);
        out1<<addInfo <<endl;
        addInfo = QStringLiteral("Herkunftsland;%1").arg(g_state->answeredCountrie);
        out1<<addInfo <<endl;
        addInfo = QStringLiteral("Mensa-Type;%1").arg(g_state->answeredMensaType);
        out1<<addInfo <<endl;
        addInfo = QStringLiteral("Ernährungsstil;%1").arg(g_state->answeredFoodType);
        out1<<addInfo <<endl;
        }
   addInfo = QStringLiteral("Essenswunsch;%1").arg(g_state->answeredDish);
   out1<<addInfo <<endl;

   file1.close();

    welcomeLabel = new QLabel();
    welcomeLabel->setText("Herzlichen Dank");
    welcomeLabel->setAlignment(Qt::AlignHCenter);

    QLabel* space = new QLabel();
    space->setText("");
    space->setAlignment(Qt::AlignHCenter);


       buttonsFirstRun = new QPushButton(tr("Weiter"));
       QObject::connect(buttonsFirstRun, SIGNAL(clicked()),this, SLOT(onActionRest()));


       initGroup = new QGroupBox(tr(""));
          initForm = new QFormLayout;
          initForm ->addRow(welcomeLabel);
          initForm ->addRow(space);
          initForm ->addRow(buttonsFirstRun);

       initGroup->setLayout(initForm);
       initGroup->setFlat(true);
       initGroup->setFixedSize(
                          1000,
                          400);
       initGroup->setAlignment(Qt::AlignCenter);

       QHBoxLayout *layout2 = new QHBoxLayout;
        layout2->addWidget(initGroup);
       layout2->setAlignment(Qt::AlignCenter);

           QWidget *window = new QWidget();
           window->setLayout(layout2);


           setCentralWidget(window);

}



void MainWindow::closeEvent(QCloseEvent *event) {
    g_state->GazeTrackingStop=true;
    g_state->GazeSettingStop=true;
    g_state->StopSlideShow=true;
    g_state = nullptr;
//    g_state->deleteLater();
}





