// Authors: Shoma Berkemeyer, Julius Schöning
// Citation: please cite the paper 'Feeling Hungry --- Association of Dietary Patterns with Food Choices Using Scene Perception' 2020 by the authors

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "mainwindow.h"

#include <QMainWindow>
#include <QtCore>
#include <QToolBar>
#include <QMenuBar>
#include <QStatusBar>
#include <QAction>

#include <QLabel> //new
#include <QFormLayout> //new
#include <QGroupBox> //new
#include <QLineEdit> //new
#include <QComboBox> //new
#include <QSpinBox> //new
#include <QPushButton> //new
#include <QGraphicsScene> //new

#include <QDockWidget>

class MainWindowUI;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0, Qt::WindowFlags flags = 0);
    ~MainWindow();

    // ==== UI STUFF ====

    // ui components
public:
    QMenu* fileMenu;
    QMenu* frameMenu;
    QMenu* toolMenu;
    QMenu* windowsMenu;
    QMenu* helpMenu;

    QLabel *cleanLabel; //new
    QLabel *welcomeLabel; //new
    QGroupBox *initGroup; //new
    QFormLayout *initForm; //new
    QGroupBox *reurnGroup; //new
    QFormLayout *returnForm; //new
    QPushButton *startTracking; //new
    QPushButton *buttonsFirstRun;//new
    QPushButton *buttonsSecondRun; //new
    QGroupBox *horizontalGroupBox;//new

    QGraphicsScene *sceneFixation;//new
    QGraphicsView *viewFixation;//new
    QGraphicsEllipseItem * gazeEllipse;//new

    QLineEdit *enterPatID; //new

    QSpinBox *integerSpinBox; //new
    QComboBox *sex; //new
    QComboBox *mensa; //new
    QComboBox *countr; //new
    QComboBox *cb; //new
    QComboBox *dish; //new



    void registerSig();


    // ==== LOGIC STUFF ====

    // actions
public:
    QAction* key;

public slots:
    void onActionRest(); //new
    void onActionNew(); //new
    void onActionStartSurvey(); //new
    void onActionContinueSurvey();//new


    void onActionStartSlideShow(); //new
    void onActionShowImage(); //new
    void onActionSaveResults(); //new

    void onActionSetUpEyeTracker();//new
    void onActionDrawGazePoint(); //new


protected:
     void closeEvent(QCloseEvent *event);

private:
    QList<QString> dishes = QList<QString>()<<QString("Bio-Spaghetti mit Bio-Soja-Bolognese")
                                            <<QString("Rindergeschnetzeltes Esterhazy in Gemüsesauce")
                                            <<QString("Gerstoni Kürbis-Curry")
                                            <<QString("Italienischer Eintopf mit Schnittbrötchen oder Salat")
                                            <<QString("Hähnchenschnitzel mit Pfirsich und Sauce Hollandaise");
    QList<QString> countries = QList<QString> () <<QString("Germany")
                                                <<QString("Afghanistan")
                                                <<QString("Aland Islands")
                                                <<QString("Albania")
                                                <<QString("Algeria")
                                                <<QString("American Samoa")
                                                <<QString("Andorra")
                                                <<QString("Angola")
                                                <<QString("Anguilla")
                                                <<QString("Antarctica")
                                                <<QString("Antigua")
                                                <<QString("Argentina")
                                                <<QString("Armenia")
                                                <<QString("Aruba")
                                                <<QString("Australia")
                                                <<QString("Austria")
                                                <<QString("Azerbaijan")
                                                <<QString("Bahamas")
                                                <<QString("Bahrain")
                                                <<QString("Bangladesh")
                                                <<QString("Barbados")
                                                <<QString("Barbuda")
                                                <<QString("Belarus")
                                                <<QString("Belgium")
                                                <<QString("Belize")
                                                <<QString("Benin")
                                                <<QString("Bermuda")
                                                <<QString("Bhutan")
                                                <<QString("Bolivia")
                                                <<QString("Bosnia")
                                                <<QString("Botswana")
                                                <<QString("Bouvet Island")
                                                <<QString("Brazil")
                                                <<QString("British Indian Ocean Trty.")
                                                <<QString("Brunei Darussalam")
                                                <<QString("Bulgaria")
                                                <<QString("Burkina Faso")
                                                <<QString("Burundi")
                                                <<QString("Caicos Islands")
                                                <<QString("Cambodia")
                                                <<QString("Cameroon")
                                                <<QString("Canada")
                                                <<QString("Cape Verde")
                                                <<QString("Cayman Islands")
                                                <<QString("Central African Republic")
                                                <<QString("Chad")
                                                <<QString("Chile")
                                                <<QString("China")
                                                <<QString("Christmas Island")
                                                <<QString("Cocos (Keeling) Islands")
                                                <<QString("Colombia")
                                                <<QString("Comoros")
                                                <<QString("Congo")
                                                <<QString("Congo, Democratic Republic of the")
                                                <<QString("Cook Islands")
                                                <<QString("Costa Rica")
                                                <<QString("Cote d'Ivoire")
                                                <<QString("Croatia")
                                                <<QString("Cuba")
                                                <<QString("Cyprus")
                                                <<QString("Czech Republic")
                                                <<QString("Denmark")
                                                <<QString("Djibouti")
                                                <<QString("Dominica")
                                                <<QString("Dominican Republic")
                                                <<QString("Ecuador")
                                                <<QString("Egypt")
                                                <<QString("El Salvador")
                                                <<QString("Equatorial Guinea")
                                                <<QString("Eritrea")
                                                <<QString("Estonia")
                                                <<QString("Ethiopia")
                                                <<QString("Falkland Islands (Malvinas)")
                                                <<QString("Faroe Islands")
                                                <<QString("Fiji")
                                                <<QString("Finland")
                                                <<QString("France")
                                                <<QString("French Guiana")
                                                <<QString("French Polynesia")
                                                <<QString("French Southern Territories")
                                                <<QString("Futuna Islands")
                                                <<QString("Gabon")
                                                <<QString("Gambia")
                                                <<QString("Georgia")
                                                <<QString("Ghana")
                                                <<QString("Gibraltar")
                                                <<QString("Greece")
                                                <<QString("Greenland")
                                                <<QString("Grenada")
                                                <<QString("Guadeloupe")
                                                <<QString("Guam")
                                                <<QString("Guatemala")
                                                <<QString("Guernsey")
                                                <<QString("Guinea")
                                                <<QString("Guinea-Bissau")
                                                <<QString("Guyana")
                                                <<QString("Haiti")
                                                <<QString("Heard")
                                                <<QString("Herzegovina")
                                                <<QString("Holy See")
                                                <<QString("Honduras")
                                                <<QString("Hong Kong")
                                                <<QString("Hungary")
                                                <<QString("Iceland")
                                                <<QString("India")
                                                <<QString("Indonesia")
                                                <<QString("Iran (Islamic Republic of)")
                                                <<QString("Iraq")
                                                <<QString("Ireland")
                                                <<QString("Isle of Man")
                                                <<QString("Israel")
                                                <<QString("Italy")
                                                <<QString("Jamaica")
                                                <<QString("Jan Mayen Islands")
                                                <<QString("Japan")
                                                <<QString("Jersey")
                                                <<QString("Jordan")
                                                <<QString("Kazakhstan")
                                                <<QString("Kenya")
                                                <<QString("Kiribati")
                                                <<QString("Korea")
                                                <<QString("Korea (Democratic)")
                                                <<QString("Kuwait")
                                                <<QString("Kyrgyzstan")
                                                <<QString("Lao")
                                                <<QString("Latvia")
                                                <<QString("Lebanon")
                                                <<QString("Lesotho")
                                                <<QString("Liberia")
                                                <<QString("Libyan Arab Jamahiriya")
                                                <<QString("Liechtenstein")
                                                <<QString("Lithuania")
                                                <<QString("Luxembourg")
                                                <<QString("Macao")
                                                <<QString("Macedonia")
                                                <<QString("Madagascar")
                                                <<QString("Malawi")
                                                <<QString("Malaysia")
                                                <<QString("Maldives")
                                                <<QString("Mali")
                                                <<QString("Malta")
                                                <<QString("Marshall Islands")
                                                <<QString("Martinique")
                                                <<QString("Mauritania")
                                                <<QString("Mauritius")
                                                <<QString("Mayotte")
                                                <<QString("McDonald Islands")
                                                <<QString("Mexico")
                                                <<QString("Micronesia")
                                                <<QString("Miquelon")
                                                <<QString("Moldova")
                                                <<QString("Monaco")
                                                <<QString("Mongolia")
                                                <<QString("Montenegro")
                                                <<QString("Montserrat")
                                                <<QString("Morocco")
                                                <<QString("Mozambique")
                                                <<QString("Myanmar")
                                                <<QString("Namibia")
                                                <<QString("Nauru")
                                                <<QString("Nepal")
                                                <<QString("Netherlands")
                                                <<QString("Netherlands Antilles")
                                                <<QString("Nevis")
                                                <<QString("New Caledonia")
                                                <<QString("New Zealand")
                                                <<QString("Nicaragua")
                                                <<QString("Niger")
                                                <<QString("Nigeria")
                                                <<QString("Niue")
                                                <<QString("Norfolk Island")
                                                <<QString("Northern Mariana Islands")
                                                <<QString("Norway")
                                                <<QString("Oman")
                                                <<QString("Pakistan")
                                                <<QString("Palau")
                                                <<QString("Palestinian Territory, Occupied")
                                                <<QString("Panama")
                                                <<QString("Papua New Guinea")
                                                <<QString("Paraguay")
                                                <<QString("Peru")
                                                <<QString("Philippines")
                                                <<QString("Pitcairn")
                                                <<QString("Poland")
                                                <<QString("Portugal")
                                                <<QString("Principe")
                                                <<QString("Puerto Rico")
                                                <<QString("Qatar")
                                                <<QString("Reunion")
                                                <<QString("Romania")
                                                <<QString("Russian Federation")
                                                <<QString("Rwanda")
                                                <<QString("Saint Barthelemy")
                                                <<QString("Saint Helena")
                                                <<QString("Saint Kitts")
                                                <<QString("Saint Lucia")
                                                <<QString("Saint Martin (French part)")
                                                <<QString("Saint Pierre")
                                                <<QString("Saint Vincent")
                                                <<QString("Samoa")
                                                <<QString("San Marino")
                                                <<QString("Sao Tome")
                                                <<QString("Saudi Arabia")
                                                <<QString("Senegal")
                                                <<QString("Serbia")
                                                <<QString("Seychelles")
                                                <<QString("Sierra Leone")
                                                <<QString("Singapore")
                                                <<QString("Slovakia")
                                                <<QString("Slovenia")
                                                <<QString("Solomon Islands")
                                                <<QString("Somalia")
                                                <<QString("South Africa")
                                                <<QString("South Georgia")
                                                <<QString("South Sandwich Islands")
                                                <<QString("Spain")
                                                <<QString("Sri Lanka")
                                                <<QString("Sudan")
                                                <<QString("Suriname")
                                                <<QString("Svalbard")
                                                <<QString("Swaziland")
                                                <<QString("Sweden")
                                                <<QString("Switzerland")
                                                <<QString("Syrian Arab Republic")
                                                <<QString("Taiwan")
                                                <<QString("Tajikistan")
                                                <<QString("Tanzania")
                                                <<QString("Thailand")
                                                <<QString("The Grenadines")
                                                <<QString("Timor-Leste")
                                                <<QString("Tobago")
                                                <<QString("Togo")
                                                <<QString("Tokelau")
                                                <<QString("Tonga")
                                                <<QString("Trinidad")
                                                <<QString("Tunisia")
                                                <<QString("Turkey")
                                                <<QString("Turkmenistan")
                                                <<QString("Turks Islands")
                                                <<QString("Tuvalu")
                                                <<QString("Uganda")
                                                <<QString("Ukraine")
                                                <<QString("United Arab Emirates")
                                                <<QString("United Kingdom")
                                                <<QString("United States")
                                                <<QString("Uruguay")
                                                <<QString("US Minor Outlying Islands")
                                                <<QString("Uzbekistan")
                                                <<QString("Vanuatu")
                                                <<QString("Vatican City State")
                                                <<QString("Venezuela")
                                                <<QString("Vietnam")
                                                <<QString("Virgin Islands (British)")
                                                <<QString("Virgin Islands (US)")
                                                <<QString("Wallis")
                                                <<QString("Western Sahara")
                                                <<QString("Yemen")
                                                <<QString("Zambia")
                                                <<QString("Zimbabwe");

};

#endif // MAINWINDOW_H
